#!/bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/../../modules/utils.sh"

# Checks if the size of the directory and 
# prints error if it is more than 90% full, warning if it's 75-90% full
# 1 Directory for which size is provided 
# 2 Size of the directory
function print_directory_size() {
  size=${2%"%"}
  if [[ "${size}" -gt "90" ]];
  then
    error "The device for the directory $1 is $2 full"
  elif [[ "${size}" -gt "75" ]];
  then
    warn "The device for the directory $1 is $2 full"
  else
    info "The device for the directory $1 is $2 full"
  fi
}

function is_rke2_server_running() {
  IS_RKE2_ACTIVE=$(systemctl is-active rke2-server)

  if [[ "${IS_RKE2_ACTIVE}" != "active" ]];
  then
    info "Errors from rke2-server logs"
    journalctl -u rke2-server | grep -i -A 2 -B 1 "level=error"
    journalctl -u rke2-server | grep -i -A 2 -B 1 "level=fatal"
    error "RKE2 Server is not running. Please re-start the rke2-server service using command : sudo systemctl restart rke2-server"
  else
    info "RKE2 Server is running."
  fi
}

function is_rke2_agent_running() {
  IS_RKE2_ACTIVE=$(systemctl is-active rke2-agent)

  if [[ "${IS_RKE2_ACTIVE}" != "active" ]];
  then
    info "Errors from rke2-agent logs"
    journalctl -u rke2-agent | grep -i -A 2 -B 1 "level=error"
    journalctl -u rke2-agent | grep -i -A 2 -B 1 "level=fatal"
    error "RKE2 Agent is not running. Please re-start the rke2-agent service using command : sudo systemctl restart rke2-agent"
  else
    info "RKE2 Agent is running."
  fi
}

# Check rancher and kubelet directory sizes
function check_directory_sizes() {
  # Check if /var/lib/rancher is mount with a separate device or check for /var/lib
  if [[ -d "/var/lib/rancher" ]];
  then
    DEVICE_NAME_CONTAINERS=$(df /var/lib/rancher | sed -n '2 p' | cut -d' ' -f1)
  else
    DEVICE_NAME_CONTAINERS=$(df /var/lib/ | sed -n '2 p' | cut -d' ' -f1)
  fi

  # Check if /var/lib/kubelet is mount with a separate device or check for /var/lib
  if [[ -d "/var/lib/kubelet" ]];
  then
    DEVICE_NAME_KUBELET=$(df /var/lib/kubelet | sed -n '2 p' | cut -d' ' -f1)
  else
    DEVICE_NAME_KUBELET=$(df /var/lib/ | sed -n '2 p' | cut -d' ' -f1)
  fi

  # If both folders are under same device, check the size to be 200GB or check for the size to be 125GB and 100GB for docker and kubelet
  if [[ "${DEVICE_NAME_CONTAINERS}" == "${DEVICE_NAME_KUBELET}" ]];
  then
    COMMON_SIZE=$(findmnt -o USE% --target /var/lib/rancher -n)
    print_directory_size "/var/lib/" "$COMMON_SIZE"
  else
    CONTAINERS_SIZE=$(findmnt -o USE% --target /var/lib/rancher -n)
    print_directory_size "/var/lib/rancher" "$CONTAINERS_SIZE"
    KUBELET_SIZE=$(findmnt -o USE% --target /var/lib/kubelet -n)
    print_directory_size "/var/lib/kubelet" "$KUBELET_SIZE"
  fi
}

function check_rke2_status() {
    export NODE_ROLE="NONE"
    [[ "$(systemctl is-enabled rke2-server 2>/dev/null)" == "enabled" ]] && export NODE_ROLE="server"
    [[ "$(systemctl is-enabled rke2-agent 2>/dev/null)" == "enabled" ]] && export NODE_ROLE="agent"

    if [[ "${NODE_ROLE}" == "server" ]];
    then
      is_rke2_server_running
    else
      is_rke2_agent_running
    fi

    if rke2 --version &>/dev/null;
    then
      info "$(rke2 --version)"
    fi
}

function is_service_active() {
    local service_name=$1

    if [ -z "$service_name" ]
    then
      echo "No argument supplied"
      return 0
    fi

    if [[ "$(systemctl is-active "$service_name")" == "active" ]];
    then
      return 0
    else
      return 1
    fi
}

function check_services_enabled() {
  if is_service_active iscsid; then info "iscsid is enabled"; else "iscsid is disable, please enable the same for longhorn to work properly 'systemctl enable iscsid && systemctl start iscsid'"; fi
  if is_service_active firewalld; then error "firewalld is enabled, plz disable the same by running 'systemctl disable firewalld'"; else info "firewalld is disabled, so we are good"; fi
  if is_service_active nm-cloud-setup.service; then error "nm-cloud-setup.service is enabled, plz disable the same by running 'systemctl disable nm-cloud-setup.service nm-cloud-setup.timer'. Ref: https://docs.rke2.io/known_issues/"; else info "nm-cloud-setup.service is disabled, so we are good"; fi
  if is_service_active nm-cloud-setup.timer; then error "nm-cloud-setup.timer is enabled, plz disable the same by running 'systemctl disable nm-cloud-setup.service nm-cloud-setup.timer'. Ref: https://docs.rke2.io/known_issues/"; else info "nm-cloud-setup.timer is disabled, so we are good"; fi
}

function main() {
    check_services_enabled
    check_rke2_status
    check_directory_sizes
}

main