#!/bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/../../../modules/utils.sh"

# Prints ceph deployment status
function ceph_deployment_status() {
    section "Ceph Health"
    toolbox_pod=$(kubectl -n rook-ceph get pod -l app=rook-ceph-tools -o jsonpath="{.items[0].metadata.name}")
    CEPH_STATUS=$(kubectl -n rook-ceph exec -i "$toolbox_pod" -- ceph status | grep health | cut -d ':' -f 2 | xargs | sed 's/\r$//')
    CEPH_REASON=$(kubectl -n rook-ceph exec -i "$toolbox_pod" -- ceph status | grep -A 1 health | sed -n 2p | xargs)
    if [[ "${CEPH_STATUS}" = "HEALTH_OK" ]];
    then
        add_to_report info "Ceph status : Healthy"
    elif [[ "${CEPH_STATUS}" = "HEALTH_WARN" ]];
    then
        add_to_report warn "Ceph status : Warning. ${CEPH_REASON}"
    else
        add_to_report error "Ceph status : Failed. ${CEPH_REASON}"
    fi

    add_to_report info "Ceph OSD Stats:"
    add_to_report execute "kubectl -n rook-ceph exec -i $toolbox_pod -- ceph osd status"

    deployments=$(kubectl -n rook-ceph get deploy --no-headers -o custom-columns=NAME:.metadata.name)
    for deploy in $deployments; do
        check_replicas "$deploy" rook-ceph
    done
}

function rabbitmq_status() {
    section "RabbitMQ Health"
    DESIRED_REPLICAS=$(kubectl -n rabbitmq get statefulset rabbitmq-server -o jsonpath="{.status.replicas}")
    READY_REPLICAS=$(kubectl -n rabbitmq get statefulset rabbitmq-server -o jsonpath="{.status.readyReplicas}")

    if [[ "${DESIRED_REPLICAS}" == "${READY_REPLICAS}" ]];
    then
        add_to_report info "RabbitMQ all replicas are up."
    else
        add_to_report warn "Not all rabbitMQ replicas are up. ReadyReplicas : ${READY_REPLICAS} DesiredReplicas : ${DESIRED_REPLICAS}"
    fi
}

function mongodb_status() {
    section "MongoDB Health"
    local NAMESPACE=mongodb

    if [ -z "$(kubectl -n $NAMESPACE get deploy -o name)" ];
    then
      add_to_report info "MongoDB is not installed, that's fine if business apps is not enabled"
      return 0
    fi

    USER_NAME=$(kubectl -n $NAMESPACE get secret mongo-admin-user -o json | jq -r '.data.USER_NAME' | base64 -d )
    PASSWORD=$(kubectl -n $NAMESPACE get secret mongo-admin-user -o json | jq -r '.data.PASSWORD' | base64 -d )
    # shellcheck disable=SC2016
    depldbstatus=$(kubectl -n $NAMESPACE exec -i mongodb-replica-set-0 -- sh -c '/usr/bin/mongo --tls --tlsCAFile /var/lib/tls/ca/ca.crt --host `hostname -f` -u '"$USER_NAME"' -p '"'"''"$PASSWORD"''"'"' --authenticationDatabase admin --eval '"'"'db.runCommand("ping").ok'"'"' --quiet' 2>/dev/null)
    # shellcheck disable=SC2016
    depldbprimary=$(kubectl -n $NAMESPACE exec -i mongodb-replica-set-0 -- sh -c '/usr/bin/mongo --tls --tlsCAFile /var/lib/tls/ca/ca.crt --host `hostname -f` -u '"$USER_NAME"' -p '"'"''"$PASSWORD"''"'"' --authenticationDatabase admin --eval '"'"'db.runCommand("hello").primary'"'"' --quiet' 2>/dev/null)

    if [ "$depldbstatus" == 1 ]; 
    then 
      add_to_report info "MongoDBConnectivity: Successful"
    else 
      add_to_report error "MongoDBConnectivity: Failed"
    fi

    if [ -n "$depldbprimary" ]; 
    then 
      add_to_report info "MongoDBPrimaryFound: True" 
    else
      add_to_report error "MongoDBPrimaryFound: False"
    fi
}

function redis_status() {
    section "Redis Health"
    DESIRED_REPLICAS=$(kubectl -n redis-system get statefulset redis-cluster -o jsonpath="{.status.replicas}")
    READY_REPLICAS=$(kubectl -n redis-system get statefulset redis-cluster -o jsonpath="{.status.readyReplicas}")

    if [[ -z "${READY_REPLICAS}" ]];
    then 
        READY_REPLICAS=0
    fi

    if [[ "${DESIRED_REPLICAS}" == "${READY_REPLICAS}" ]];
    then
        add_to_report info "Redis all replicas are up."
    else
        add_to_report warn "Not all Redis replicas are up. ReadyReplicas : ${READY_REPLICAS} DesiredReplicas : ${DESIRED_REPLICAS}"
    fi
}

function smb_driver_status() {
    section "SMB Driver Health"

    SMB_DRIVER_STATUS=$(kubectl -n argocd get Application csi-driver-smb --ignore-not-found -o jsonpath="{.status.health.status}")

    if [[ -z "${SMB_DRIVER_STATUS}" ]];
    then
      add_to_report info "Smb driver not enabled."
    else
      CONTROLLER_DESIRED_REPLICAS=$(kubectl -n smb-driver get deployment csi-smb-controller -o jsonpath="{.status.replicas}")
      CONTROLLER_READY_REPLICAS=$(kubectl -n smb-driver get deployment csi-smb-controller -o jsonpath="{.status.readyReplicas}")

      if [[ "${CONTROLLER_DESIRED_REPLICAS}" == "${CONTROLLER_READY_REPLICAS}" ]];
      then
          add_to_report info "Smb driver controller all replicas are up."
      else
          add_to_report warn "Not all smb driver controller replicas are up. ReadyReplicas : ${CONTROLLER_READY_REPLICAS} DesiredReplicas : ${CONTROLLER_DESIRED_REPLICAS}"
      fi

      NODE_DESIRED_SCHEDULED=$(kubectl -n smb-driver get daemonset csi-smb-node -o jsonpath="{.status.desiredNumberScheduled}")
      NODE_NUMBER_READY=$(kubectl -n smb-driver get daemonset csi-smb-node -o jsonpath="{.status.numberReady}")

      if [[ "${NODE_DESIRED_SCHEDULED}" == "${NODE_NUMBER_READY}" ]];
      then
          add_to_report info "Smb driver node daemon all replicas are up."
      else
          add_to_report warn "Not all smb driver node deamon replicas are up. NumberReady : ${NODE_NUMBER_READY} DesiredScheduled : ${NODE_DESIRED_SCHEDULED}"
      fi
    fi
}

function alert_status() {
    section "Prometheus Alerts"
    cluster_ip=$(kubectl get svc rancher-monitoring-prometheus -n cattle-monitoring-system -o jsonpath='{.spec.clusterIP}') || true
    active_alerts=$(curl "http://${cluster_ip}:9090/api/v1/alerts") || true
    filtered_result=""
    echo "$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "warning")))' <<< $active_alerts)" | jq -c '.[]' | while read -r alert; 
        do add_to_report warn "Active alert: $alert"; 
    done

    echo "$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "critical")))' <<< $active_alerts)" | jq -c '.[]' | while read -r alert; 
        do add_to_report error "Active alert: $alert"; 
    done
}

alert_status
redis_status
ceph_deployment_status
rabbitmq_status
mongodb_status
smb_driver_status