#!/bin/bash
# Tools needed in container : s3cmd:2.1.0, bc, zip, gzip

DIR_LOCATION="/data/support-bundle"

info() {
  echo "[INFO] [$(date +'%Y-%m-%dT%H:%M:%S%z')]: $*"
}

warn() {
  echo "[WARN] [$(date +'%Y-%m-%dT%H:%M:%S%z')]: $*" >&2
}

error() {
  echo -e "\e[0;31m[ERROR][$(date +'%Y-%m-%dT%H:%M:%S%z')]:\e[0m $*" >&2
  exit 1
}

function display_usage() {
  echo "usage: $(basename "$0") [-hd] -i infile -o outfile -S S3Host -P S3Port -A S3AccessKey -K S3SecretKey -N namespaces -F startdate -D numberofdays -B bucketName"
  echo "  -d          Run in debug"
  echo "  -h          Display help"
  echo "  -S          AWS_HOST value for S3 storage"
  echo "  -P          AWS_PORT value for S3 storage"
  echo "  -A          AWS_ACCESS_KEY value for S3 storage"
  echo "  -K          AWS_SECRET_KEY value for S3 storage"
  echo "  -N          Comma separated list of namespaces from which want to collect the data"
  echo "  -B          S3 Bucket name that holds the logs from the cluster"
  echo "  -F          Start date to collect the data, YYYY-MM-DD format. If start date is not mentioned, it will collect logs from last 7 days. Ex 2021-06-06"
  echo "  -D          Number of days from the start date for which we want to collect the logs (default value is 7 days)"
}

# Parse args utility
function parse_args() {
  while getopts 'hdi:o:S:P:A:K:N:B:F:D:' flag "$@"; do
    case "${flag}" in
    d)
      info "Running in debug"
      set -x
      ;;
    h)
      display_usage
      exit 0
      ;;
    S)
      export AWS_HOST=${OPTARG}
      ;;
    P)
      export AWS_PORT=${OPTARG}
      ;;
    A)
      export AWS_ACCESS_KEY=${OPTARG}
      ;;
    K)
      export AWS_SECRET_KEY=${OPTARG}
      ;;
    N)
      export NAMESPACES=${OPTARG}
      ;;
    B)
      export BUCKET_NAME=${OPTARG}
      ;;
    F)
      export FROM_DATE=${OPTARG}
      ;;
    D)
      export NUMBER_OF_DAYS=${OPTARG}
      ;;
    :)
      error "Invalid option: ${OPTARG} requires an argument."
      ;;
    *)
      error "Unexpected option ${flag}"
      ;;
    esac
  done
  shift $((OPTIND - 1))
  if [ -z "$FROM_DATE" ];
  then
    FROM_DATE=$(date --date="6 day ago" +%Y-%m-%d)
    NUMBER_OF_DAYS=7
  fi

  if [ -z "${NUMBER_OF_DAYS}" ];
  then
    NUMBER_OF_DAYS=7
  fi
}

# Set s3cmd context passing following params
# 1 Host of the object store
# 2 Port of the object store
# 3 Access key of the user
# 4 Secret key of the user
function setS3CMDContext() {
  OBJECT_GATEWAY_INTERNAL_HOST=$AWS_HOST
  OBJECT_GATEWAY_INTERNAL_PORT=$AWS_PORT
  export AWS_HOST=$OBJECT_GATEWAY_INTERNAL_HOST
  export AWS_ENDPOINT=$OBJECT_GATEWAY_INTERNAL_HOST:$OBJECT_GATEWAY_INTERNAL_PORT
  export AWS_ACCESS_KEY_ID=$AWS_ACCESS_KEY
  export AWS_SECRET_ACCESS_KEY=$AWS_SECRET_KEY

  if ! s3cmd ls --host="${AWS_HOST}" --no-ssl;
  then
    warn "Ceph S3 store is not responding. Exiting and collecting logs directly from pods."
    exit 0
  else
    info "Ceph S3 store responding, collecting logs from there"
  fi
}

# Gets kubernetes objects yaml files 
# 1 Namespace
# 2 ObjectType - pod, deployment, daemonset etc
function getKubernetesObjects() {
  ns=$1
  objectType=$2
  info "Getting $objectType from $ns namespace"
  OBJECTS=$(kubectl -n "$ns" get "$objectType" --no-headers -o custom-columns=NAME:.metadata.name)
  for obj in $OBJECTS; do
    mkdir -p "$DIR_LOCATION/$objectType/$ns/"
    kubectl -n "$ns" get "$objectType" "$obj" -o yaml > "$DIR_LOCATION/$objectType/$ns/$obj.yaml"
  done
}

# Uploads $SERVICE_BUNDLE_ZIP to the S3 storage
function uploadSupportBundle() {
    DATE=$(date +'%Y_%m_%d')
    # upload the zip file to storage
    s3cmd mb s3://support-bundles/ --host="${AWS_HOST}" --host-bucket= --no-ssl
    s3cmd put "/data/$SERVICE_BUNDLE_ZIP"  "s3://support-bundles/$DATE/" --host="${AWS_HOST}" --host-bucket= --no-ssl

    # get the download link for the uploaded bundle
    SUPPORT_BUNDLE_URL=$(s3cmd signurl "s3://support-bundles/$DATE/$SERVICE_BUNDLE_ZIP" "$(echo "$(date +%s) + 3600 * 24 * 7" | bc)" --host="${AWS_HOST}" --host-bucket= --no-ssl)
}

# Consolidate all the logs podwise
function consolidateLogs() {
  LOGS_DIRECTORY="$DIR_LOCATION/logs"
  # Logs structure is /date/namespace/pod.container/logsfiles
  all_folders=$(ls .)
  for folder in $all_folders
  do
    if [ -d "$folder" ];
    then
      info "Processing logs for the date $folder"
      cd "$folder" || exit
      namespaces=$(ls .)
      for ns in $namespaces
      do
        if [[ "$NAMESPACES" != *"$ns"* ]];
        then
          # If namespace is not in the list provided, skip it
          continue
        fi
        info "Processing $ns namespace logs"
        if [ -d "$ns" ];
        then
          cd "$ns" || exit
          pods=$(ls .)
          for pod in $pods
          do
            if [ -d "$pod" ];
            then
              cd "$pod" || exit
              pod_name=$(echo "$pod" | cut -d '.' -f 1)
              container_name=$(echo "$pod" | cut -d '.' -f 2)
              mkdir -p "$LOGS_DIRECTORY/$ns/$pod_name"
              LOG_LOCATION=$LOGS_DIRECTORY/$ns/$pod_name/$container_name.log
              all_chunks=$(ls .)
              # For each container, append logs to the file at location /support-bundle/logs/namespace/pod/container.logs
              for chunk in $all_chunks
              do
                  gzip -d "$chunk"
                  file_name=$(echo "$chunk" | cut -d '.' -f 1)
                  cat "$file_name" >> "$LOG_LOCATION"
              done
              cd ..
            fi
          done
          cd ..
        fi
      done
      cd ..
    fi
  done
}


# Collect firing alerts from Prometheus
function collectClusterAlerts() {
  mkdir -p ${DIR_LOCATION}/alerts/
  info "Getting active alerts from Prometheus"
  cluster_ip=$(kubectl -n cattle-monitoring-system get services/rancher-monitoring-prometheus -o jsonpath='{.spec.clusterIP}') || error "Failed to get cluster_ip for rancher-monitoring-prometheus"
  active_alerts=$(curl "http://${cluster_ip}:9090/api/v1/alerts") || error "Curl failed"
  filtered_alerts="$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "warning")))' <<< $active_alerts)" 
  echo "${filtered_alerts}" > ${DIR_LOCATION}/alerts/warning.json
  filtered_alerts="$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "critical")))' <<< $active_alerts)" 
  echo "${filtered_alerts}" > ${DIR_LOCATION}/alerts/critical.json
}

# Collect rest of the kuberntes object details, status of deployments, jobs, etc
function collectClusterDetails() {
  collectClusterAlerts || error "Failed to fetch active alerts"
  # Get current logs from the pods and different objects yaml files to see the status and any events
  echo "$NAMESPACES" | tr "," "\n" | while read -r ns
  do
    info "Processing objects in $ns namespace"

    # Get logs from the current states of the pods for the last 4 hours
    PODS=$(kubectl -n "$ns" get pods --no-headers -o custom-columns=NAME:.metadata.name)
    info "Getting current logs from $ns namespace"
    for pod in $PODS; do
      CONTAINERS=$(kubectl -n "$ns" get pod "$pod" -o jsonpath="{.spec.containers[*].name}")
      for container in $CONTAINERS; do
        mkdir -p "$DIR_LOCATION/current-logs/$ns/$pod/"
        kubectl -n "$ns" logs "$pod" "$container" --since=4h >> "$DIR_LOCATION/current-logs/$ns/$pod/$container.log" || echo "The container $pod/$container logs could not be collected"
      done
      mkdir -p "$DIR_LOCATION/pods/$ns/"
      kubectl -n "$ns" get pod "$pod" -o yaml > "$DIR_LOCATION/pods/$ns/$pod.yaml"
    done
    
    # Get Kubernets yaml files of different Objects. Deployments, jobs, daemonsets
    getKubernetesObjects "$ns" "deployments"

    getKubernetesObjects "$ns" "jobs"

    getKubernetesObjects "$ns" "daemonsets"

    getKubernetesObjects "$ns" "services"

    # Get events from the namespaces
    mkdir -p $DIR_LOCATION/events/
    kubectl -n "$ns" get events -o json | jq 'del(.items[].metadata.managedFields)' > "$DIR_LOCATION/events/$ns.json"
  done
  # get the nodes details
  kubectl describe nodes > $DIR_LOCATION/nodes.info

  echo "Support bundle generated from ceph store" > ${DIR_LOCATION}/rook-ceph-up.txt

  # collect Identity Service support bundle 
  uipathNs="uipath"
  mkdir -p "$DIR_LOCATION/configmaps/$uipathNs"
  for identity_cm in "identity-service" "identity-service-clients" "identity-service-customconfig"
  do 
    kubectl -n "$uipathNs" get cm "$identity_cm" -o yaml > "$DIR_LOCATION/configmaps/$uipathNs/$identity_cm.yaml"
  done
}

# Get the Redis Enterprise support bundle
function getRedisSupportBundle(){
  local bundle_folder="${DIR_LOCATION}/redis_enterprise"
  local namespace="redis-system"
  local container="redis-enterprise-node"
  local log_file
  local pods

  info "Generating the Redis Enterprise support bundle"

  mkdir -p "${bundle_folder}" || true
  pods="$(kubectl get pods -n ${namespace} -l "redis.io/role=node" --no-headers -o custom-columns=NAME:.metadata.name)"
  for pod in ${pods}
  do 
    info "Getting the redis support bundle from pod: ${pod}"
    kubectl exec -n "${namespace}" "${pod}" -c "${container}" -- /opt/redislabs/bin/debuginfo &>/dev/null || true
    log_file="$(kubectl exec -n "${namespace}" "${pod}" -c "${container}" -- ls -t /tmp | grep debuginfo | head -1)"
    kubectl cp -c "${container}" ${namespace}/"${pod}":/tmp/"${log_file}" "${bundle_folder}/${pod}.tar.gz" || true
  done
}

function main() {
  parse_args "$@"

  # Set S3CMD Context, by loading env variables
  setS3CMDContext

  info "Bucket name is $BUCKET_NAME"

  # Directory where all logs will be collected
  mkdir -p $DIR_LOCATION/logs

  # Temp directory where logs will be downloaded to
  mkdir -p /data/service-fabric-logs

  cd /data/service-fabric-logs || exit

  # Download logs for specified dates. From start date till number of days
  LOGS_DATE=$(date -d "$FROM_DATE")
  counter=0
  while [[ $counter -lt "$NUMBER_OF_DAYS" ]];
  do
    day=$(date -d "$LOGS_DATE + $counter days" +%Y_%m_%d)
    echo "$NAMESPACES" | tr "," "\n" | while read -r ns
    do
      mkdir -p "$day/$ns/"
      s3cmd get "s3://$BUCKET_NAME/$day/$ns/" "$day/$ns/" --host="${AWS_HOST}" --no-ssl --host-bucket= --recursive
    done
    counter=$((counter+1)) 
  done

  # Consolidate all the logs at one place
  consolidateLogs

  # Collect rest of the kubernetes object details
  collectClusterDetails

  # Get the Redis Enterprise support bundle
  getRedisSupportBundle

  # Create a zip file out of the logs folder
  SERVICE_BUNDLE_ZIP="service-fabric-logs-$(date +'%Y-%m-%d-%H-%M-%S').zip"
  cd /data || return
  zip "/data/$SERVICE_BUNDLE_ZIP" support-bundle/* -r
  cd ..

  # Upload support bundle
  uploadSupportBundle

  echo "SupportBundleLink $SUPPORT_BUNDLE_URL"
}

main "$@"